#include "mex.h"
#include "matrix.h"

/* This MATLAB mex function can concatenate four 8-bit chars to make 
 one 32-bit float.  This function may be useful when sending data from a
 microcontroller UART to MATLAB when your packets do not have a strict
 structure or byte length and 'fread' is not flexible enough.
 
 Usage: float_out = fourChar2Float([ char3 char2 char1 char0])
    This will give you a floating point equivalent to the concatentation of
    the four bytes.
    |------32-bit float-----|
    |char3|char2|char1|char0|
 
 Compile with "mex 'fourChar2Float.c'"
 */

void chars2float(float outputFloat[], unsigned char parts[4]){
	union packetFloat{
		float dataFloat;
		unsigned char dataChar[4];
	} data;
    data.dataChar[3] = parts[0];
    data.dataChar[2] = parts[1];
    data.dataChar[1] = parts[2];
    data.dataChar[0] = parts[3];
    outputFloat[0] = data.dataFloat;
}


void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] )
{
  float *floatOut;

  /* Create matrix for the return argument. */

  plhs[0] = mxCreateNumericMatrix(1,1,mxSINGLE_CLASS, mxREAL);
  
  /* Assign pointers to each input and output. */

  floatOut = mxGetPr(plhs[0]);  
  /* Call the subroutine. */
  chars2float(floatOut, mxGetPr(prhs[0]));
}
